<?php
/**
 * Frontend class
 *
 * @author  Yithemes
 * @package YITH WooCommerce EU Energy Label
 * @version 1.1.1
 */

if ( ! defined( 'YITH_WCEUE' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'YITH_WCEUE_Frontend' ) ) {
	/**
	 * Frontend class.
	 * The class manage all the Frontend behaviors.
	 *
	 * @author   Leanza Francesco <leanzafrancesco@gmail.com>
	 * @since    1.0.0
	 */
	class YITH_WCEUE_Frontend {

		/**
		 * Single instance of the class
		 *
		 * @var \YITH_WCEUE_Frontend
		 * @since 1.0.0
		 */
		protected static $_instance;

		public $is_in_sidebar = false;

		public $show_on_product = false;
		public $show_in_shop    = false;

		private $theme_info = false;

		/**
		 * Returns single instance of the class
		 *
		 * @return \YITH_WCEUE_Frontend
		 * @since 1.0.0
		 */
		public static function get_instance() {
			$self = __CLASS__ . ( class_exists( __CLASS__ . '_Premium' ) ? '_Premium' : '' );

			return ! is_null( $self::$_instance ) ? $self::$_instance : $self::$_instance = new $self;
		}

		/**
		 * Constructor
		 *
		 * @access public
		 * @since  1.0.0
		 */
		protected function __construct() {
			// add frontend css
			add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

			$this->add_energy_label_in_position();

			// action to set this->is_in_sidebar
			add_action( 'dynamic_sidebar_before', array( $this, 'set_is_in_sidebar' ) );
			add_action( 'dynamic_sidebar_after', array( $this, 'unset_is_in_sidebar' ) );

			add_shortcode( 'yith_eu_energy_label', array( $this, 'render_yith_eu_energy_label_shortcode' ) );

			$this->show_in_shop = 'yes' === get_option( 'yith-wceue-show-in-shop-thumb', 'no' );

			$theme_info = $this->get_theme_info();
			if ( $theme_info->slug === 'yith-proteo' ) {
				// YITH Proteo support!
				add_action( 'woocommerce_before_shop_loop_item_title', array( $this, 'energy_container_start' ), 9 );
				add_action( 'woocommerce_before_shop_loop_item_title', array( $this, 'energy_container_end' ), 20 );
			} else {
				add_filter( 'woocommerce_product_get_image', array( $this, 'add_energy_label_on_shop_thumb' ), 10, 2 );
			}
		}

		/**
		 * Get the current theme info
		 *
		 * @return stdClass
		 * @since 1.1.15
		 */
		public function get_theme_info() {
			if ( ! $this->theme_info ) {
				$this->theme_info          = new stdClass();
				$this->theme_info->name    = '';
				$this->theme_info->slug    = '';
				$this->theme_info->version = '';

				$current_theme = wp_get_theme();
				if ( $current_theme ) {
					if ( $current_theme->parent() ) {
						$current_theme = $current_theme->parent();
					}
					$this->theme_info->name    = $current_theme->get( 'Name' );
					$this->theme_info->slug    = sanitize_title( strtolower( $this->theme_info->name ) );
					$this->theme_info->version = $current_theme->get( 'Version' );
				}
			}

			return $this->theme_info;
		}

		/**
		 * Start the container for wrapping the badge
		 *
		 * @see   YITH_WCEUE_Frontend::energy_container_end()
		 * @since 1.1.15
		 */
		public function energy_container_start() {
			ob_start();
		}

		/**
		 * Print buffered content to wrap the badge
		 *
		 * @see   YITH_WCEUE_Frontend::energy_container_start()
		 * @since 1.1.15
		 */
		public function energy_container_end() {
			global $product;
			$html = ob_get_clean();
			$html = $this->add_energy_label_on_shop_thumb( $html, $product );
			echo $html;
		}

		/**
		 * render [yith_eu_energy_label] shortcode
		 *
		 * @return string
		 * @since 1.1.3
		 */
		public function render_yith_eu_energy_label_shortcode() {
			ob_start();
			$this->print_energy_label();

			return ob_get_clean();
		}

		/**
		 * Add Energy Label on shop thumbnails
		 *
		 * @param string     $thumb
		 * @param WC_Product $product
		 *
		 * @return string
		 */
		public function add_energy_label_on_shop_thumb( $thumb, $product ) {
			$show = ! $this->is_in_sidebar && ! is_cart() && ! is_checkout();
			$show = is_archive() ? ( $show && $this->show_in_shop ) : $show;
			$show = is_product() ? ( $show && $this->show_on_product ) : $show;

			$show = apply_filters( 'yith_wceue_show_energy_label_on_product', $show, $product->get_id() );

			if ( $show ) {
				return $this->show_energy_on_product( $thumb, $product->get_id() );
			} else {
				return $thumb;
			}
		}

		/**
		 * @param $thumb
		 *
		 * @return string
		 * @since 1.1.3
		 */
		public function show_energy_on_product_thumb( $thumb ) {

			global $product;

			if ( ! did_action( 'woocommerce_product_thumbnails' ) && $product ) {
				$print_directly = false;
				$div_close      = '</div>';
				if ( version_compare( WC()->version, '3.0', '>=' ) && get_theme_support( 'wc-product-gallery-slider' ) ) {
					$_val = $thumb;
					$_val = rtrim( $_val );
					if ( strrpos( $_val, $div_close ) === strlen( $_val ) - strlen( $div_close ) ) {
						$print_directly = true;
						$thumb          = $_val;
					}
				}

				if ( $print_directly ) {
					$thumb = substr( $thumb, 0, - strlen( $div_close ) );
					$thumb .= $this->get_energy_label_html( $product );
					$thumb .= $div_close;
				} else {
					$thumb = $this->show_energy_on_product( $thumb, $product->get_id() );
				}

			}

			return $thumb;
		}

		/**
		 * Edit image in products
		 *
		 * @access public
		 *
		 * @param string $thumb product image
		 * @param int    $post_id
		 *
		 * @return string
		 * @since  1.0.0
		 * @author Leanza Francesco <leanzafrancesco@gmail.com>
		 */
		public function show_energy_on_product( $thumb, $post_id ) {
			$product = wc_get_product( $post_id );

			if ( ! $product || strpos( $thumb, 'yith-wceue-thumb-and-energy-container' ) > 0 ) {
				return $thumb;
			}

			$energy_label_html = $this->get_energy_label_html( $product );

			if ( ! $energy_label_html ) {
				return $thumb;
			}

			$container = "<div class='yith-wceue-thumb-and-energy-container'>";
			$container .= $thumb;
			$container .= $energy_label_html;
			$container .= "</div><!--yith-wceue-thumb-and-energy-container-->";

			return $container;
		}

		/**
		 * @param WC_Product $product
		 *
		 * @return mixed
		 */
		public function get_energy_label_html( $product ) {

			/* Compatibility with WooCommerce Show Single Variations (by Iconic) plugin */
			if ( $product instanceof WC_Product_Variation ) {
				$product = wc_get_product( $product->get_parent_id() );
			}

			$meta         = $product->get_meta( '_yith_wceue_eu_meta', true );
			$energy_label = $product->get_meta( '_yith_wceue_eu_energy_label', true );
			$html         = '';

			if ( $energy_label ) {
				$t_args = array(
					'class'        => 'yith-wceue-eu-energy-label-mini',
					'energy_label' => $energy_label,
					'product_id'   => $product->get_id(),
					'meta'         => $meta,
				);
				ob_start();
				wc_get_template( '/energy_label.php', $t_args, '', YITH_WCEUE_TEMPLATE_PATH );
				$html = ob_get_clean();
			}

			return $html;
		}


		/**
		 * Add Energy Label in position set in Settings Tab
		 *
		 * @access   public
		 * @since    1.0.0
		 * @author   Leanza Francesco <leanzafrancesco@gmail.com>
		 */
		public function add_energy_label_in_position() {
			$button_position = get_option( 'yith-wceue-position-in-single-product', 'before_summary' );
			$bp_array        = array();
			switch ( $button_position ) {
				case 'on_thumb':
					add_filter( 'woocommerce_single_product_image_thumbnail_html', array( $this, 'show_energy_on_product_thumb' ), 10, 2 );
					$this->show_on_product = true;
					break;
				case 'before_summary':
					$bp_array = array( 'action' => 'woocommerce_before_single_product_summary', 'priority' => 25 );
					break;
				case 'before_description':
					$bp_array = array( 'action' => 'woocommerce_single_product_summary', 'priority' => 15 );
					break;
				case 'after_description':
					$bp_array = array( 'action' => 'woocommerce_single_product_summary', 'priority' => 25 );
					break;
				case 'after_add_to_cart':
					$bp_array = array( 'action' => 'woocommerce_single_product_summary', 'priority' => 35 );
					break;
				case 'after_summary':
					$bp_array = array( 'action' => 'woocommerce_after_single_product_summary', 'priority' => 9 );
					break;
			}
			if ( $bp_array ) {
				add_action( $bp_array['action'], array( $this, 'print_energy_label' ), $bp_array['priority'] );
			}
		}

		/**
		 * Print Best Sellers Icon
		 *
		 * @access   public
		 * @since    1.0.0
		 * @author   Leanza Francesco <leanzafrancesco@gmail.com>
		 */
		public function print_energy_label() {
			global $product;

			$base_product_id = yit_get_base_product_id( $product );

			$meta         = get_post_meta( $base_product_id, '_yith_wceue_eu_meta', true );
			$energy_label = get_post_meta( $base_product_id, '_yith_wceue_eu_energy_label', true );
			$t_args       = array(
				'class'        => '',
				'energy_label' => $energy_label,
				'product_id'   => $base_product_id,
				'meta'         => $meta,
			);
			wc_get_template( '/energy_label.php', $t_args, YITH_WCEUE_TEMPLATE_PATH, YITH_WCEUE_TEMPLATE_PATH );
		}

		/**
		 * Set this->is in sidebar to true
		 *
		 * @access public
		 * @since  1.1.4
		 * @author Leanza Francesco <leanzafrancesco@gmail.com>
		 */
		public function set_is_in_sidebar() {
			$this->is_in_sidebar = true;
		}

		/**
		 * Set this->is in sidebar to false
		 *
		 * @access public
		 * @since  1.1.4
		 * @author Leanza Francesco <leanzafrancesco@gmail.com>
		 */
		public function unset_is_in_sidebar() {
			$this->is_in_sidebar = false;
		}


		public function enqueue_scripts() {
			wp_enqueue_style( 'yith_wceue_frontend_style', YITH_WCEUE_ASSETS_URL . '/css/frontend.css' );
		}
	}
}
/**
 * Unique access to instance of YITH_WCEUE_Frontend class
 *
 * @return YITH_WCEUE_Frontend|YITH_WCEUE_Frontend_Premium
 * @since 1.0.0
 */
function YITH_WCEUE_Frontend() {
	return YITH_WCEUE_Frontend::get_instance();
}